//
// d o s - c o n s o l e . c
//
// DOS console module
//
// Sep. 11, 2002 by W.Nishida (http://www.skyfree.org)
//

asm(".code16gcc");

// Character print

void printc(char ch) {
  asm("
	pushl	%edx			# Save current EDX
	movl	8( %ebp ), %edx		# Get character code
	movb	$2, %ah			# Output a character
	int	$0x21			# Do function call
	popl	%edx			# Restore EDX
  ");
 }

// String print

void prints(char* str) {
  while (*str)
    printc(*str++);
 }

// Character input

int getchar(void) {
  asm("
	xorl	%eax, %eax
	movb	$1, %ah			# Keyboard input
	int	$0x21			# Do function call
	xorb	%ah, %ah		# EAX holds character code
  ");
 }

// Dump byte as hexadecimal format

void dump_byte(int data) {
  char hex[] = "0123456789ABCDEF";

  printc(hex[ (data & 0xF0) >> 4 ]);    // Output higher nibble
  printc(hex[ data & 0x0F ]);           // Output lower nibble
 }

// Dump word as hexadecimal format

void dump_word(int data) {

  dump_byte((data & 0xFF00) >> 8);	// Output higher byte
  dump_byte(data & 0xFF);		// Output lower byte
 }

// Dump long as hexadecimal format

void dump_long(int data) {

  dump_word((data & 0xFFFF0000) >> 16);	// Output higher word
  dump_word(data & 0xFFFF);		// Output lower word
 }

// Dump 16 bytes of specified segment:offset address space

void dump_memory(int segment, int offset) {

  dump_word(segment);			// Output segment address
  printc(':');
  dump_word(offset);			// Output offset address
  printc(' ');

  asm("
    pushl	%esi			# Save current ESI
    pushw	%fs			# Save current FS

    pushl	%esi			# Save current ESI
    pushw	%fs			# Save current FS
    movl	8(%ebp), %eax		# Get segment address
    movw	%ax, %fs		# Change FS
    movl	12(%ebp), %esi		# Get offset address
    movl	$16, %ecx		# Set counter (16 bytes)
  1:
    xorl	%eax, %eax		# Celar EAX
    movb	%fs:(%esi), %al		# Load byte data
    inc		%esi			# Forward address pointer
    pushl	%esi			# Save ESI
    pushl	%ecx			# Save ECX
    pushl	%eax			# Set character code
    call	dump_byte		# dump_byte()
    popl	%eax			# Adjust ESP
    movb	' ', %al		# Set space code
    pushl	%eax			# Set character code
    call	printc			# printc(ch)
    popl	%eax			# Adjust ESP
    popl	%ecx			# Recover ECX
    popl	%esi			# Recover ESI
    loop	1b			# Repeat

    popw	%fs			# Restore FS
    popl	%esi			# Restore ESI
   ");

  prints("\r\n");
 }
