//
// k b c . h
//
// Header file for the Keyboard controller module
//
// Sep. 10, 2002 by W.Nishida (http://www.skyfree.org)
//

// 8042 keyboard controler (KBC) definition

#define KBC_CTRL	0x64	// 8042 KBC control port (command/status)
#define KBC_DATA	0x60	// 8042 KBC data buffer
#define IOWAIT		0x80	// 1 miscrosec wait / access

#define KBC_OBF		0x01	// Output buffer is full (please read)
#define KBC_IBF		0x02	// Input buffer is full (don't write)
#define KBC_ABF		0x20	// AUX output buffer is full
#define KBC_BUSY	(KBC_OBF | KBC_IBF)

#define KBC_PORT2_RESET	1	// P20 bit is connected to CPU reset
#define KBC_PORT2_A20	2	// P21 bit is connected to A20 gate

// 8042 command bytes

#define KBC_GET_CTRL	0x20	// Read current 8042 control mode
#define KBC_SET_CTRL	0x60	// Setup 8042 control mode
#define KBC_READ_PORT2	0xD0	// Read 8042 port2
#define KBC_WRITE_PORT2	0xD1	// Write 8042 port2

#define KBC_CTRL_XLT	0x40	// Translate keyboard scan codes

// Device command bytes

#define KBC_WRITE_LED	0xED	// Write LED
#define KBC_SET_SCAN	0xF0	// Set scan code set
#define KBC_SET_REPEAT	0xF3	// Setup keyboard repeat rate
#define KBC_ACK		0xFA	// ACK answer code

#define KBC_SCAN_READ	0	// Read current scan code set
#define KBC_SCAN_1	1	// Activate scan code set1
#define KBC_SCAN_2	2	// Activate scan code set2
#define KBC_SCAN_3	3	// Activate scan code set3

// 8259 programmable interrupt controller (PIC) definition

#define PIC_MIMR	0x21	// Master PIC IMR

#define PIC_IMR_KEY     2       // Master IMR keyboard IRQ bit

// Function prototypes

int  inb(int port);
void outb(int data, int port);
int  kbc_status(void);
void kbc_command(int cmd);
int  kbc_read(void);
void kbc_write(int data);
void disable_keyint(void);
void enable_keyint(void);
